####################################################################
#
# Licensed Materials - Property of IBM
# pidnumber 5724-P55, 5724-P57, 5724-P58, 5724-P59
#  Copyright IBM Corporation 2007. All Rights Reserved.
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
####################################################################

#####################################################################################
#
# Silverstream install functions
#
#####################################################################################

install_ss()
{
    SetDefaultValues
    ReadPrevConfig
    create_silverstream_home
    check_available_space
    check_oracle_connectivity
    create_aginstdefs

    # run the perl script that installs silverstream
    install_silverstream
    errcode=$?
    if [ ! ${errcode} -eq 0 ]; then
	log "Can't install SilverStream!"
	gracefulexit ${errcode}
    fi

    # start the server
    debug "starting the server";
    silverstream_init startnodb

    # stop the server.
    debug "stopping the server";
    silverstream_init stop
}

upgrade_ss()
{
    SetDefaultValues
    ReadPrevConfig

    # stop the server.
    debug "stopping the server";
    silverstream_init stop

    check_oracle_connectivity

    # run the perl script that installs silverstream
    install_silverstream
    errcode=$?
    if [ ! ${errcode} -eq 0 ]; then
	log "Can't install SilverStream!"
	gracefulexit ${errcode}
    fi

    # start the server
    debug "starting the server";
    silverstream_init startnodb

    # stop the server.
    debug "stopping the server";
    silverstream_init stop
}

uninstall_ss()
{
    # read config
    SetDefaultValues
    ReadPrevConfig

    # stop silverstream
    silverstream_init stop

    # remove the SS deployment directory (ex. /opt/silverstream)
    runcmd errcode 10 "rm -rf $SS_HOME" "Can't remove $SS_HOME!"
}

apply_novell_patch()
{
    log "Applying a Novell patch to SilverStream is not implemented"
}

# check oracle connectivity and run DVOptimizerToRule.sql
check_oracle_connectivity()
{
    status "Checking oracle connectivity ..."
    check_tnsnames ${ORACLE_HOME} ${ORA_SILVERMASTER_TNS}
    status "running DVOptimizerToRule.sql ..."
    if [ "${ORA_VERSION}" = "9" ]; then optimize_oracle9; fi 
}

# create silverstream deployment directory: defaults to /opt/silverstream
create_silverstream_home()
{
    status "Creating directory ${SS_HOME} ..."
    typeset CMD="mkdir -p ${SS_HOME}"
    runcmd errcode 10 "$CMD" "Can't create silverstream home directory ${SS_HOME}!"
    set_user_perms "${SS_HOME}"
}

# Create the configuration file that feeds SilverStream with
# the proper configuration. Name: <installation directory>/.aginstdefs
create_aginstdefs()
{
    log "Writing installation config to ${SS_HOME}/${SS_CONFIG_FILE}"
    ORACTNS=`echo $ORA_SILVERMASTER_TNS | tr [:lower:] [:upper:]`
    if [ $ACTION_TYPE = "install" ]; then
	RUNSMINIT="yes"
	DO_UPGRADE="no"
    else
	RUNSMINIT="no"
	DO_UPGRADE="yes"
    fi
    log "\
# SilverStream re-installation defaults file
# This is a generated file.,
# DO NOT EDIT THIS FILE.
dbtype = Oracle$ORA_VERSION
masterName = $ORA_SILVERMASTER_TNS
dbUsername = $ORA_SILVERMASTER_USER
dbPassword = $ORA_SILVERMASTER_PASSWORD
DbHost = $ORA_SILVERMASTER_TNS
oraclehome = $ORACLE_HOME
serviceport = $SS_NAMESERVICE_PORT
httport = $SS_HTTP_PORT
licenseStr = C6632800000001F3E22D05192006
installdocs = no
installsamples = no
oracthin = 1
oracsyn = no
oractns = $ORACTNS
adminuser = $SS_ADMIN_USER
adminpasswd = $SS_ADMIN_PASSWORD
orac_zip_a_dir = $ORACLE_HOME/jdbc/lib
runSmInit = $RUNSMINIT
doUpgrade = $DO_UPGRADE
restrictserver = yes
" > ${SS_HOME}/${SS_CONFIG_FILE}

    set_user_perms "${SS_HOME}/${SS_CONFIG_FILE}"
}

# run the silverstream install perl script
install_silverstream()
{
    status "Run the silverstream install perl script ..."
    typeset ERRCODE=0
    typeset PERLCMD="(cd ${SS_BUNDLE}; ./perl.ag '$PWD/install.pl' invokefromsh '$PWD' '$SS_BUNDLE' '$SS_HOME')"
    typeset CMD="su - ${UNIX_SS_USER} -c \"$PERLCMD\" 2>&1 >> $LOGFILE"
    if [ $EXEC = "YES" ]; then
	debug "$CMD"
	if [ "$DEBUG" = "ON" ]; then CMD="$CMD"
	else CMD="$CMD >> ${LOGFILE} 2>&1"
	fi
	eval "$CMD"
	ERRCODE=$?
    else
	log "PERLCMD = $PERLCMD"
    fi

    return $ERRCODE
}

set_user_perms()
{
    FILEPATH=$1

    typeset CMD="chown ${UNIX_SS_USER}:${UNIX_DV_GROUP} ${FILEPATH}"
    runcmd errcode 10 "$CMD" "Can't change ownership of ${FILEPATH}!"

    typeset CMD="chmod 775 ${FILEPATH}"
    runcmd errcode 10 "$CMD" "Can't change permissions of ${FILEPATH}!"
}

check_available_space()
{
    # a silverstream installation needs 328M - round up for comfort (unit is KB)
    typeset SPACE_NEEDED=400000
    typeset AVAILABLE_SPACE
    status "Calculating the amount of disk space available for SilverStream ..."
    typeset CMD="AVAILABLE_SPACE=`df -k ${SS_HOME} | awk '{print $4}' | xargs | awk '{print $2}'`"
    runcmd errcode 10 "$CMD" "Can't get disk free space of ${SS_HOME} with df!"
    debug "There are ${AVAILABLE_SPACE} kilobytes available on ${SS_HOME}"
    debug "Total space needed for SilverStream install is ${SPACE_NEEDED} kilobytes"

    if [ "${SPACE_NEEDED}" -gt "${AVAILABLE_SPACE}" ]; then
	error "There is not enough space to install SilverStream on ${SS_HOME}."
        error "${SPACE_NEEDED} kilobytes are needed, and only ${AVAILABLE_SPACE} are available!"
        gracefulexit 51
    else
        debug "There is enough space to install SilverStream on ${SS_HOME}."
        status ok
    fi
}
